"""
erasus.selectors.ensemble — Ensemble selection methods.
"""

from erasus.selectors.ensemble.voting import VotingSelector
from erasus.selectors.ensemble.weighted_fusion import WeightedFusionSelector
